#!/usr/bin/python -u

import argparse
import os
import sys

import errno
from builtins import input
from conf.ConfigManager import ConfigManager

# from EzADB import EzADB
from ezadb.EzADB import EzADB
from ezadb import __version__

PERMISSION_TYPE_GRANT = 'GRANT'
PERMISSION_TYPE_REVOKE = 'REVOKE'
PERMISSIONS_LIST = [PERMISSION_TYPE_GRANT, PERMISSION_TYPE_REVOKE]

parser = argparse.ArgumentParser(description="eZadb ADB - use adb like a boss!",
                                 epilog="GNU General Public License v3.0")

# default group options
main_group = parser.add_argument_group('default options')
sub_maingroup = main_group.add_mutually_exclusive_group()
sub_maingroup.add_argument('-d', '--devices', action='store_true')
sub_maingroup.add_argument('-t', '--inputText', dest='inputText')
sub_maingroup.add_argument('-u', '--uninstall', dest='uninstall',
                           help='Uninstall a specific application indicating the package name')
sub_maingroup.add_argument('-i', '--install', dest='install',
                           help='Install a specific application indicating the full path of the apk')
sub_maingroup.add_argument('-sc', '--screenshot_path', dest="screenshot_path",
                           help='Take a ScreenShot of a device display')
sub_maingroup.add_argument('-url', dest="url_link",
                           help='Opens a specific URL in the device')

group = parser.add_argument_group('permissions', description='Define permissions of an application')
group.add_argument('--package', help='Package name of the application')
group.add_argument('--permission', help='Android Permission to be set')
group.add_argument('--type', help='Type of permission', choices=PERMISSIONS_LIST)
parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__,
                    help='Print the version number and exit')
args = parser.parse_args()

configManager = ConfigManager()
ezADB = EzADB(configManager.get_adb_path())


def show_app_menu(apps_list):
    """Show the list of applications installed in the device"""
    os.system('clear')
    print(30 * "-", "Select the Application", 30 * "-")
    menu = {}
    for index, app in enumerate(apps_list):
        menu[index] = app
    # add an exit option
    menu[len(menu)] = "Exit"
    loop = True
    while loop:
        for index, app in enumerate(menu):
            if index == len(menu) - 1:
                print("[{}]: Exit ".format(index))
            else:
                print("[{}]: Application: {}".format(index, menu[index]))
        choice = int(input("Enter your choice[{}-{}]:".format(0, (len(menu) - 1))))

        if choice > (len(menu) - 1):
            print("\n Not Valid Choice Try again")
        elif choice == (len(menu) - 1):
            sys.exit(0)
            # This will make the while loop to end as not value of loop is set to False
        else:
            return menu[choice]


def ask_adb_path():
    """Ask the user to input the full path of the ADB file"""

    full_path = input("Please enter the ADB full path (e.g.: ~/android-sdk/platform-tools/adb): ")
    if os.path.exists(full_path):
        configManager.set_adb_path(full_path)
    else:
        print("The file ADB path entered does not exist")
        sys.exit(errno.ENOENT)


def main():
    # check if the adb is defined
    if not configManager.get_adb_path():
        ask_adb_path()

    if len(sys.argv[1:]) == 0:
        parser.print_help()
        parser.exit()

    if len(ezADB.get_devices()) > 0:
        if args.devices:
            print('List of devices attached')
            devices = ezADB.get_devices()
            for idx, device in enumerate(devices):
                print("[{}]:Device: {} \n".format(idx, device))
        elif args.inputText:
            print(ezADB.input_text(args.inputText))
        elif args.uninstall:
            print(ezADB.uninstall_app(None, args.uninstall))
        elif args.install:
            print(ezADB.installAPK(None, args.install))
        elif args.screenshot_path:
            ezADB.get_screenshot(None, args.screenshot_path)
        elif args.package or args.permission or args.type:
            if all([args.permission, args.type]):
                application = args.package
                # no application defined retrieve from the menu
                if application is None:
                    application = show_app_menu(ezADB.get_applications_installed())
                permission_type = EzADB.Permission.GRANT if (
                    args.type.lower() == PERMISSION_TYPE_GRANT.lower()) else EzADB.Permission.REVOKE
                ezADB.set_permission(application, args.permission, permission_type)
            else:
                raise parser.error("Package, permission and type all must be set ")
        elif args.url_link:
            ezADB.open_url(args.url_link)
    else:
        print('No devices attached')


if __name__ == "__main__":
    main()



import os

from .Config import Config
from ezadb.utils.Log import Log

try:
    from shutil import which
except ImportError:
    from backports.shutil_which import which

ADB_DEFAULT_NAME = 'adb'
DEFAULT_CONFIG_FOLDER = ".ezadb/config"
DEFAULT_CONFIG_FILENAME = "config.cfg"


class ConfigManager:
    def __init__(self):
        self.userDir = os.path.expanduser('~')
        config_folder = os.path.normpath("{userpath}/{default_folder}".format(userpath=self.userDir,
                                                                              default_folder=DEFAULT_CONFIG_FOLDER))
        self.configFilePath = config_folder + os.path.sep + DEFAULT_CONFIG_FILENAME
        # create the folders if they don't exist
        if not os.path.exists(config_folder):
            os.makedirs(config_folder)

        self.config = Config(self.configFilePath)

        # try to set the adb if needed
        self._find_adb_path()

    def set_adb_path(self, path):
        """Set the path for the adb executable"""

        default_section = self.config.get_section('default')
        default_section.set('ADB_PATH', path)
        self.config.save_changes()

    def get_adb_path(self):
        """Get the adb executable path"""

        return self.config.get_section('default').get('ADB_PATH')

    def _find_adb_path(self):
        # check if we already defined the adb path
        if self.get_adb_path() is None:
            # try to find the adb executable in the path
            adb_full_path = which(ADB_DEFAULT_NAME)
            if adb_full_path is not None:
                self.set_adb_path(adb_full_path)
        else:
            Log.info(self.__class__, "ADB path already defined {path}".format(path=self.get_adb_path()))



import os
from pygubu.stockimage import StockImage, StockImageException

# Initialize images
APP_DIR = path = os.path.abspath(
    os.path.join(os.path.dirname(os.path.abspath(__file__)), ".."))
IMAGES_DIR = os.path.join(APP_DIR, "images")
StockImage.register_from_dir(IMAGES_DIR)


class Utils:
    @staticmethod
    def replace_all(text, dic):
        """"Returns a string replacing all the sequence of characters defined in the dictionary . """
        for key, value in dic.items():
            text = text.replace(key, value)
        return text

    @staticmethod
    def set_app_icon(tk_top_level):
        try:
            tk_top_level.wm_iconname('ezadb')
            tk_top_level.tk.call('wm', 'iconphoto', tk_top_level._w, StockImage.get('ezadb'))
        except StockImageException as e:
            pass



import subprocess

from conf.SystemUtils import SystemUtils
from utils.Log import Log
from utils.Utils import Utils

try:
    from enum import Enum
    from shutil import which
except ImportError:
    from enum import Enum
    from backports.shutil_which import which


class EzADB:
    def __init__(self, adb_path):
        # try to find the adb executable in the path
        if adb_path:
            self.__fullADBPath__ = SystemUtils.os_based_path(adb_path)

    def get_devices(self):
        """  Retrieve the list of connected devices

            Returns:
                the list of android devices that are attached """
        result = self.__execute_adb_cmd__('devices').decode()
        result = Utils.replace_all(result, {'\r': '', 'package:': ''}).splitlines()
        # pop the title item
        result.pop(0)
        result = list(filter(None, result))
        # remove the word device
        result = [w.replace('device', '') for w in result]
        return result

    def get_applications_installed(self):
        result = self.__execute_adb_cmd__('shell pm list packages').decode().replace('package:', '')
        result = result.splitlines()
        # pop the title item
        result.pop(0)
        result = list(filter(None, result))
        return result

    def installAPK(self, device_id=None, apk_full_path=None):
        """Performs the installation of an APK file"""
        if apk_full_path is None:
            Log.warn(self.__class__, 'The supplied path of apk is invalid')
            return

        command = 'install {apkpath}'.format(apkpath=apk_full_path)

        result = self.__execute_adb_cmd__(command)
        Log.info(self.__class__, result, device_id)

    def uninstall_app(self, device_id=None, package_name=None):
        """Performs the uninstall of an application """
        if package_name is None:
            Log.warn(self.__class__, 'The supplied PackageName is invalid')
            return

        command = "uninstall {package}".format(package=package_name)

        result = self.__execute_adb_cmd__(command, device_id)
        Log.info(self.__class__, result.decode())

    def enable_disable_app(self, device_id, package_name, app_state):
        """Enables/Disables a certain application/component """
        if package_name is None:
            Log.warn(self.__class__, 'The supplied PackageName is invalid')
            return

        # Type checking
        if not isinstance(app_state, EzADB.State):
            raise TypeError('appState must be an instance of State Enum')

        command = "shell pm {state} {package}" \
            .format(state="enable" if EzADB.State.ENABLED == app_state else "disable", package=package_name)

        result = self.__execute_adb_cmd__(command, device_id)
        Log.info(self.__class__, result.decode())

    def get_app_state(self, device_id=None, package_name=None):
        """Check is a certain application is enabled"""
        if package_name is None:
            Log.warn(self.__class__, 'The supplied PackageName is invalid')
            return

        command = "shell pm list packages -e {package}".format(package=package_name)

        result = self.__execute_adb_cmd__(command, device_id).decode()
        filtered_apps = Utils.replace_all(result, {'\r': '', 'package:': ''}).splitlines()

        if package_name in filtered_apps:
            return EzADB.State.ENABLED
        else:
            return EzADB.State.DISABLED

    def input_text(self, text):
        """Inserts a certain text string in the application selected field
               currently running on the device"""
        return self.__execute_adb_cmd__('shell input text ' + text)

    def open_url(self, url):
        """Opens a certain

             :param url: the url that should be open """
        return self.__execute_adb_cmd__('shell am start -a android.intent.action.VIEW -d ' + url)

    def set_permission(self, package_name, permission_name, perm_type):
        """ Grants or Revokes a certain permission for a specific application"""
        # Type checking
        if not isinstance(perm_type, EzADB.Permission):
            raise TypeError('permission must be an instance of Permission Enum. Received:{invalid}'
                            .format(invalid=perm_type))

        if perm_type == EzADB.Permission.GRANT:
            permission_type = 'grant'
        else:
            permission_type = 'revoke'

        return self.__execute_adb_cmd__("shell pm {permissionType} {package} {permissionName} "
                                        .format(permissionType=permission_type, package=package_name,
                                                permissionName=permission_name))

    def pull_file(self, file_name, target_dir, device_id):
        """Pulls a specific file from the device"""

        command = "pull {file_path} {target_path}".format(file_path=file_name, target_path=target_dir)
        Log.info(self.__class__, command)
        return self.__execute_adb_cmd__(command, device_id, True)

    def pull_apk(self, package_name, target_dir, device_id):
        """Pulls the APK of a certain application from the device"""
        apk_full_path = self._get_apk_full_path(device_id, package_name)
        return self.pull_file(apk_full_path, target_dir, device_id)

    def reboot_device(self, device_id):
        """Reboots a specific device"""
        return self.__execute_adb_cmd__("reboot", device_id)

    def shutdown_device(self, device_id):
        """Shutdown a specific device"""
        return self.__execute_adb_cmd__("shell reboot -p", device_id)

    def app_clear_data(self, device_id, package_name):
        """Clear the application of a specific application"""
        self.__execute_adb_cmd__("shell pm clear {app}".format(app=package_name), device_id)

    def get_screenshot(self, device_id, target_dir):
        """ Take a ScreenShot of a device display"""
        # create a unique temp filename based on the timestamp
        temp_filename = '/sdcard/{random_name}'.format(random_name=SystemUtils.generate_random_filename('png'))
        command = 'shell screencap -p {filename}'.format(filename=temp_filename)

        # make the screen dump to the device
        self.__execute_adb_cmd__(command, device_id, True)
        # pull the file from the device to the target dir selected
        self.pull_file(temp_filename, target_dir, device_id)
        # deletes the temp file from the device
        self._delete_file(temp_filename, device_id)

    def get_app_info(self, device_id, package_name, full_info=False):
        """Get system state associated with the given PACKAGE"""
        if package_name is None:
            Log.warn(self.__class__, 'The supplied PackageName is invalid')
            return
        if full_info:
            command = "shell pm dump {package}".format(package=package_name)
        else:
            command = "shell dumpsys package {package}".format(package=package_name)
        return self.__execute_adb_cmd__(command, device_id)

    def __execute_adb_cmd__(self, command, device_id=None, ignore_output=False):
        """ Executes a certain adb command

            :param command: the adb command to be executed """

        # append device id if exists
        if device_id is not None:
            command = '-s {device} {cmd}'.format(device=device_id, cmd=command)

        # start the adb server if needed
        self._start_adb_server()

        # remove spaces and split command into a list
        args = command.rstrip().split()
        # insert the adb path to the beginning of the list
        args.insert(0, self.__fullADBPath__)

        if ignore_output:
            proc = subprocess.Popen(args)
            proc.wait()
        else:
            return subprocess.check_output(args)

    def _delete_file(self, file_path, device_id):
        """ Deletes a specific file from the device"""
        self.__execute_adb_cmd__('shell rm {file_path}'.format(file_path=file_path), device_id, True)

    def _start_adb_server(self):
        """Check whether the adb server process is running."""
        subprocess.call([self.__fullADBPath__, 'start-server'])

    def _get_apk_full_path(self, device_id, package_name):
        apk_full_path = self.__execute_adb_cmd__(
            "shell pm path {package}".format(package=package_name), device_id) \
            .decode().replace('package:', '').rstrip()
        return apk_full_path

    class Permission(Enum):
        """ Enum for the Application permissions"""
        GRANT = 0
        REVOKE = 1

    class State(Enum):
        """ Enum for the Application state"""
        DISABLED = 0
        ENABLED = 1



import gettext
import locale
import os

#  The translation files will be under
#  @LOCALE_DIR@/@LANGUAGE@/LC_MESSAGES/@APP_NAME@.mo
#
APP_NAME = "ezadb"

# Not sure in a regular desktop:
#

project_basedir = path = os.path.abspath(
    os.path.join(os.path.dirname(os.path.abspath(__file__)), ".."))


APP_DIR = project_basedir
LOCALE_DIR = os.path.join(APP_DIR, 'locale')

# Now we need to choose the language. We will provide a list, and gettext
# will use the first translation available in the list
#
#  In maemo it is in the LANG environment variable
#  (on desktop is usually LANGUAGES)
#
DEFAULT_LANGUAGES = os.environ.get('LANG', '').split(':')
DEFAULT_LANGUAGES += ['en_US']

# Try to get the languages from the default locale
languages = []
lc, encoding = locale.getdefaultlocale()
if lc:
    languages = [lc]

# Concat all languages (env + default locale),
#  and here we have the languages and location of the translations
#
languages += DEFAULT_LANGUAGES
mo_location = LOCALE_DIR

# Lets tell those details to gettext
#  (nothing to change here for you)
gettext.install(True)
gettext.bindtextdomain(APP_NAME, mo_location)
gettext.textdomain(APP_NAME)
language = gettext.translation(APP_NAME,
                               mo_location,
                               languages=languages,
                               fallback=True)

Translator = language.gettext

# And now in your modules you can do:
#
# import i18n
# _ = i18n.language.gettext
#



#!/usr/bin/python
import os
import threading
import webbrowser

import pygubu

from ezadb import __version__
from ezadb import __AUTHOR__
from ezadb.EzADB import EzADB
from ezadb.conf.ConfigManager import ConfigManager
from ezadb.conf.I18n import Translator
from ezadb.conf.SystemUtils import SystemUtils
from ezadb.gui.PreviewImageDialog import PreviewImageDialog
from ezadb.utils.Log import Log
from ezadb.utils.Utils import Utils

try:
    import tkinter as tk  # for python 3
    from tkinter import *
    from tkinter import ttk
    from tkinter import filedialog
    import tkinter.font as tkFont

except:
    import Tkinter as tk  # for python 2
    import tkMessageBox as messagebox

# translator function
_ = Translator

# default app url
DEFAULT_URL = 'https://github.com/filipebatista/eZadb'


class EzADBGui(pygubu.TkApplication):
    def __init__(self, master):
        self.selected_device = None
        self.progress_dialog = None
        self.about_dialog = None
        self.appinfo_dialog = None
        self.configManager = ConfigManager()
        self.ezADB = EzADB(self.configManager.get_adb_path())
        pygubu.TkApplication.__init__(self, master)

    def _create_ui(self):
        # Create a builder
        self.builder = builder = pygubu.Builder(Translator)

        # Load an ui file
        builder.add_from_file(os.path.join(os.path.dirname(__file__), 'design_main.ui'))

        # Create the widget using a master as parent
        self.mainwindow = builder.get_object('mainwindow', self.master)
        toplevel = self.master.winfo_toplevel()

        # Set main menu
        self.mainmenu = menu = builder.get_object('main_menu', toplevel)
        toplevel['menu'] = menu

        # Create ProgressBar
        s = ttk.Style()
        # s.theme_use("default")

        # custom properties for the progressbar
        s.configure("TProgressbar", thickness=15)

        # devices connected
        self.tree_devices = builder.get_object('treeview_devices', self.master)

        # devices connected item listener
        self.tree_devices.bind('<ButtonRelease-1>', self.on_treecolumn_device_click)
        self.tree_devices.bind('<Button-3>', self.rightClickCopyMenu)

        # device related buttons
        self.btn_reboot = builder.get_object('btn_device_reboot', self.master)
        self.btn_shutdown = builder.get_object('btn_device_shutdown', self.master)
        self.btn_screenshot = builder.get_object('btn_device_screenshot', self.master)

        # app related buttons
        self.btn_install = builder.get_object('btn_app_install', self.master)
        self.btn_uninstall = builder.get_object('btn_app_uninstall', self.master)
        self.btn_clear_data = builder.get_object('btn_app_clear_data', self.master)
        self.btn_pull_apk = builder.get_object('btn_app_pull_apk', self.master)
        self.btn_app_disable = builder.get_object('btn_app_disable', self.master)
        self.btn_app_info = builder.get_object('btn_app_info', self.master)

        # installed apps
        self.treeview_apps = builder.get_object('treeview_apps', self.master)
        self.treeview_apps.bind('<ButtonRelease-1>', self.on_treecolumn_apps_click)
        self.treeview_apps.bind('<Button-3>', self.rightClickCopyMenu)

        # set the callbacks
        builder.connect_callbacks(self)

        # set the app icon
        Utils.set_app_icon(toplevel)

    def _init_after(self):
        # initializes the value combo box only if we already have the adb path
        if self.configManager.get_adb_path() is not None:
            self.init_devices()
        else:
            self.ask_adb_path()

    def _create_progress_dialog(self):
        builder = pygubu.Builder(Translator)
        builder.add_from_file(os.path.join(os.path.dirname(__file__), 'progress_dialog.ui'))
        dialog = builder.get_object(
            'progressdialog', self.toplevel)
        # set dialog icon
        Utils.set_app_icon(dialog.toplevel)
        progressbar = builder.get_object('progressbar1')
        progressbar.start()
        return dialog

    def _center_dialog(self, dialog):
        """Adjust the dialog to center of parent window"""
        if dialog:
            # calculate middle point
            x = self.mainwindow.winfo_rootx()
            y = self.mainwindow.winfo_rooty()
            x = x + (self.mainwindow.winfo_width() / 2) - (dialog.toplevel.winfo_reqwidth() / 2)
            y = y + (self.mainwindow.winfo_height() / 2) - (dialog.toplevel.winfo_reqheight() / 2)
            geometry = '+{0}+{1}'.format(int(x), int(y))
            dialog.toplevel.geometry(geometry)

    def show_progress_dialog(self):
        def show():
            if self.progress_dialog is None:
                self.progress_dialog = self._create_progress_dialog()
                self.progress_dialog.run()
            else:
                self.progress_dialog.show()
            self._center_dialog(self.progress_dialog)

        self.mainwindow.after(100, show)

    def hide_progress_dialog(self):
        def hide():
            if self.progress_dialog:
                self.progress_dialog.close()

        self.mainwindow.after(100, hide)

    def ask_adb_path(self, initial_dir="/"):
        initial_dir = initial_dir if initial_dir is not None else "/"
        adbpath = filedialog.askopenfilename(title=_("Select ADB executable folder"),
                                             initialdir=os.path.dirname(initial_dir),
                                             parent=self.mainwindow)
        if adbpath:
            self.configManager.set_adb_path(adbpath)
            self.ezADB = EzADB(self.configManager.get_adb_path())
            self.init_devices()

    def init_devices(self):
        self.show_progress_dialog()
        if self.treeview_apps.get_children():
            self.treeview_apps.delete(*self.treeview_apps.get_children())
        if self.tree_devices.get_children():
            self.tree_devices.delete(*self.tree_devices.get_children())
        threading.Thread(target=self.load_devices).start()

    def load_devices(self):
        # clear previous list of Devices and Apps
        # self.delete_all_treeview_items(self.treeview_apps)
        # self.delete_all_treeview_items(self.tree_devices)
        devices = self.ezADB.get_devices()
        for val in devices:
            self.tree_devices.insert('', tk.END, iid=val, text=val, values=val, tags='treedevices')
        self.hide_progress_dialog()

    def on_treecolumn_device_click(self, event):
        item_selected = self.get_selected_tree_item(self.tree_devices)
        if item_selected:
            self.selected_device = item_selected
            self.enable_buttons([self.btn_reboot, self.btn_shutdown, self.btn_screenshot, self.btn_install])
            self.show_progress_dialog()
            threading.Thread(target=self.load_installed_apps).start()

    def on_treecolumn_apps_click(self, event):
        item_selected = self.get_selected_tree_item(self.treeview_apps)
        if item_selected:
            self.enable_buttons([self.btn_clear_data, self.btn_uninstall,
                                 self.btn_pull_apk, self.btn_app_disable, self.btn_app_info])
            self.show_progress_dialog()
            threading.Thread(target=self.get_app_state, args=[item_selected]).start()

    def get_app_state(self, item_selected):
        # invert the disable button text according to app current state
        if self.ezADB.get_app_state(self.selected_device, item_selected) == EzADB.State.ENABLED:
            app_disable_text = _('Disabled')
        else:
            app_disable_text = _('Enabled')

        def __callback__():
            self.btn_app_disable['text'] = app_disable_text
            self.hide_progress_dialog()

        self.mainwindow.after(100, __callback__)

    def on_reboot_click(self):
        self.ezADB.reboot_device(self.selected_device)

    def on_shutdown_click(self):
        self.ezADB.shutdown_device(self.selected_device)

    def on_uninstall_click(self):
        app_selected = self.get_selected_tree_item(self.treeview_apps)
        device_selected = self.get_selected_tree_item(self.tree_devices)
        self.ezADB.uninstall_app(device_selected, app_selected)
        self.on_treecolumn_device_click(None)

    def on_install_click(self):
        apk_file_path = filedialog.askopenfilename(title=_("Select APK to install"),
                                                   filetypes=(
                                                       ("Android Package File", "*.apk"), (_("All Files"), "*.*")),
                                                   parent=self.mainwindow)
        if apk_file_path:
            self.show_progress_dialog()
            threading.Thread(target=self.install_apk, args=[apk_file_path]).start()

    def install_apk(self, apk_file_path):
        self.ezADB.installAPK(self.selected_device, apk_file_path)
        Log.debug(self.__class__, "[on_install_click] {filepath}".format(filepath=apk_file_path))

        def __callback__():
            # reload the apps list
            self.on_treecolumn_device_click(None)
            self.hide_progress_dialog()

        self.mainwindow.after(100, __callback__)

    def on_cleardata_click(self):
        app_selected = self.get_selected_tree_item(self.treeview_apps)
        device_selected = self.get_selected_tree_item(self.tree_devices)
        self.ezADB.app_clear_data(device_selected, app_selected)
        self.on_treecolumn_device_click(None)

    def on_pull_apk_click(self):
        item_selected = self.get_selected_tree_item(self.treeview_apps)
        if item_selected:
            default_filename = str(item_selected).replace('.', '_') + ".apk"
            file_path = filedialog.asksaveasfilename(title=_('Save as...'),
                                                     initialfile=default_filename,
                                                     defaultextension=".apk",
                                                     filetypes=(
                                                         ("Android Package File", "*.apk"), (_("All Files"), "*.*")))
            self.show_progress_dialog()
            threading.Thread(target=self.pull_apk, args=(item_selected, file_path)).start()

    def on_app_menuitem_clicked(self, itemid):
        if itemid == 'subitem_settings':
            self.ask_adb_path(self.configManager.get_adb_path())
        if itemid == 'subitem_refresh_devices':
            self.init_devices()

    def on_screenshot_click(self):
        if self.selected_device:
            file_path = filedialog.asksaveasfilename(title=_("Save as..."),
                                                     initialfile=SystemUtils.generate_random_filename('png'),
                                                     defaultextension=".png",
                                                     filetypes=(
                                                         ("Portable Network Graphics", "*.png"),
                                                         (_("All Files"), "*.*")))
            self.show_progress_dialog()
            threading.Thread(target=self.get_screenshot, args=[file_path]).start()

    def on_disable_click(self):
        app_selected = self.get_selected_tree_item(self.treeview_apps)
        device_selected = self.get_selected_tree_item(self.tree_devices)
        current_app_state = self.ezADB.get_app_state(device_selected, app_selected)

        self.ezADB.enable_disable_app(device_selected, device_selected,
                                      EzADB.State.ENABLED if EzADB.State.DISABLED == current_app_state
                                      else EzADB.State.ENABLED)

    def on_show_info_click(self):
        app_selected = self.get_selected_tree_item(self.treeview_apps)
        self.show_progress_dialog()
        threading.Thread(target=self.get_app_info, args=[app_selected]).start()

    @staticmethod
    def get_selected_tree_item(tree_view):
        item_selected = tree_view.selection()
        if item_selected:
            return item_selected[0]
        return None

    def delete_all_treeview_items(self, tree_view):
        if tree_view:
            x = tree_view.get_children()
            Log.info(self.__class__, 'get_children values: ', x, '\n')
            if x != '()' or x != '\n':  # checks if there is something in the first row
                for child in x:
                    tree_view.delete(child)

    @staticmethod
    def enable_buttons(buttons):
        for btn in buttons:
            btn.config(state=tk.NORMAL)

    def load_installed_apps(self):
        # clear previous list if has any
        if self.treeview_apps.get_children():
            self.treeview_apps.delete(*self.treeview_apps.get_children())

        # get apps are installed in the device
        installed_apps = self.ezADB.get_applications_installed()
        for val in installed_apps:
            if val:
                self.treeview_apps.insert('', tk.END, iid=val, text=val, values=val, tags='treeapps')
        self.hide_progress_dialog()

    def pull_apk(self, app_selected, target_path):
        self.ezADB.pull_apk(app_selected, target_path, self.selected_device)
        self.hide_progress_dialog()
        Log.info(self.__class__, '[pull_apk]:{app}=>{target}'.format(app=app_selected, target=target_path))

    def get_screenshot(self, target_path):
        if target_path:
            self.ezADB.get_screenshot(self.selected_device, target_path)
            self.hide_progress_dialog()
            self.show_img_preview(target_path)

    def get_app_info(self, selected_app):
        self.show_progress_dialog()
        fullinfo = self.ezADB.get_app_info(self.selected_device, selected_app, True)
        lightinfo = self.ezADB.get_app_info(self.selected_device, selected_app)

        def __callback__():
            self.hide_progress_dialog()
            self.show_app_info_dialog(selected_app, fullinfo, lightinfo)

        self.master.after(100, __callback__)

    def show_img_preview(self, file_path):
        def open_preview():
            dialog = PreviewImageDialog(self.toplevel, file_path)
            self.mainwindow.wait_window(dialog.top)

        self.mainwindow.after(100, open_preview)

    def rightClickCopyMenu(self, event):
        """action in event of button 3 on tree view"""
        treeview = event.widget

        def copy_to_clipboard():
            self.master.clipboard_clear()
            # text to clipboard
            text_to_copy = self.get_selected_tree_item(treeview)
            self.master.clipboard_append(text_to_copy)

        menu = Menu(self.mainwindow, tearoff=0)
        menu.add_command(label="Copy", command=copy_to_clipboard)

        # select row under mouse
        iid = treeview.identify_row(event.y)
        if iid:
            # mouse pointer over item
            treeview.selection_set(iid)
            menu.post(event.x_root, event.y_root)
        else:
            # mouse pointer not over item
            # occurs when items do not fill frame
            # no action required
            pass

    # Help menu
    def on_help_menuitem_clicked(self, itemid):
        if itemid == 'help_online':
            webbrowser.open_new_tab(DEFAULT_URL)
        elif itemid == 'help_about':
            self.show_about_dialog()

    def show_about_dialog(self):
        if self.about_dialog is None:
            self.about_dialog = self._create_about_dialog()
            self.about_dialog.run()
        else:
            self.about_dialog.show()
        self._center_dialog(self.about_dialog)

    def _create_about_dialog(self):
        builder = pygubu.Builder(Translator)
        builder.add_from_file(os.path.join(os.path.dirname(__file__), 'about_dialog.ui'))

        dialog = builder.get_object('about_dialog', self.master.winfo_toplevel())
        Utils.set_app_icon(dialog.toplevel)
        # set version
        entry = builder.get_object('version')
        txt = entry.cget('text')
        txt = txt.replace('%version%', str(__version__))
        entry.configure(text=txt)
        # set authors
        entry = builder.get_object('copyr')
        txt = entry.cget('text')
        txt = txt.replace('%user1%', __AUTHOR__[0]['Name'])
        entry.configure(text=txt)

        def on_about_close_click():
            dialog.close()

        builder.connect_callbacks({'on_about_close_click': on_about_close_click})
        return dialog

    def show_app_info_dialog(self, selected_app, fullinfo, lightinfo):
        dialog_exists = self.appinfo_dialog is not None
        builder = pygubu.Builder(Translator)
        # create the dialog in the first time
        if not dialog_exists:
            builder.add_from_file(os.path.join(os.path.dirname(__file__), 'appinfo_dialog.ui'))
            self.appinfo_dialog = builder.get_object('appinfo_dialog', self.master.winfo_toplevel())
            Utils.set_app_icon(self.appinfo_dialog.toplevel)

        # set dialog title
        self.appinfo_dialog.toplevel.title('AppInfo for: ' + selected_app)

        # set the dialog with the application information
        text_fullinfo = builder.get_object('text_fullinfo')
        text_lightinfo = builder.get_object('text_lightinfo')
        # enable the controls
        text_fullinfo.config(state=NORMAL)
        text_lightinfo.config(state=NORMAL)
        # clear the previous text
        text_fullinfo.delete(1.0, tk.END)
        text_lightinfo.delete(1.0, tk.END)
        # add the new text
        text_fullinfo.insert(tk.END, fullinfo)
        text_lightinfo.insert(tk.END, lightinfo)

        def __show_select_copy_menu__(event):
            # define the call back for item click
            def __callback__(itemid):
                if itemid == 'copy':
                    self.master.clipboard_clear()
                    # text to clipboard
                    text_to_copy = event.widget.get(SEL_FIRST, SEL_LAST)
                    self.master.clipboard_append(text_to_copy)
                elif itemid == 'select_all':
                    """Select all text in the text widget"""
                    event.widget.tag_add(SEL, "1.0", END)
                    event.widget.focus_set()

            # create the menu to show
            menu = Menu(event.widget, tearoff=0)
            menu.add_command(label="Copy", command=lambda: __callback__('copy'))
            menu.add_command(label="Select All", command=lambda: __callback__('select_all'))
            menu.post(event.x_root, event.y_root)

        # make sure the widget gets focus when clicked
        # on, to enable highlighting and copying to the
        # clipboard.
        text_fullinfo.bind("<1>", lambda event: text_fullinfo.focus_set())
        text_lightinfo.bind("<1>", lambda event: text_lightinfo.focus_set())

        text_fullinfo.bind('<Button-3>', __show_select_copy_menu__)
        text_lightinfo.bind('<Button-3>', __show_select_copy_menu__)

        # disable the controls
        text_fullinfo.config(state=DISABLED)
        text_lightinfo.config(state=DISABLED)

        if not dialog_exists:
            self.appinfo_dialog.run()
        else:
            self.appinfo_dialog.show()
        self._center_dialog(self.appinfo_dialog)



import logging
import os
from logging.handlers import RotatingFileHandler

# Initialize logger
main_logger = logging.getLogger('ezADB')
main_logger.setLevel(logging.DEBUG)
DEFAULT_LOG_FOLDER = os.path.expanduser('~') + "/.ezadb/logs/"
DEFAULT_LOG_FILENAME = 'ezADB.log'
LOG_FILE_PATH = DEFAULT_LOG_FOLDER + DEFAULT_LOG_FILENAME

# create the folders if they don't exist
if not os.path.exists(DEFAULT_LOG_FOLDER):
    os.makedirs(DEFAULT_LOG_FOLDER)

# Add the log message handler to the logger
rotatingHandler = RotatingFileHandler(LOG_FILE_PATH, maxBytes=2000, backupCount=100)
rotatingHandler.setLevel(logging.DEBUG)

# create a logging format
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
rotatingHandler.setFormatter(formatter)

main_logger.addHandler(rotatingHandler)


class Log:
    @staticmethod
    def debug(cls, *text):
        Log._get_logger_(cls).debug(text)

    @staticmethod
    def info(cls, *text):
        Log._get_logger_(cls).info(text)

    @staticmethod
    def error(cls, *text):
        Log._get_logger_(cls).error(text)

    @staticmethod
    def warn(cls, *text):
        Log._get_logger_(cls).warning(text)

    @staticmethod
    def _get_logger_(_class_):
        if _class_ is None:
            class_logger = main_logger
        else:
            class_logger = logging.getLogger(_class_.__name__)

        # ensure that the logger has an handler
        if not class_logger.handlers:
            class_logger.setLevel(logging.DEBUG)
            class_logger.addHandler(rotatingHandler)
        return class_logger



import platform
import sys
from ezadb.gui.EzADBGui import EzADBGui
from ezadb.utils.Log import Log

try:
    import tkinter as tk  # for python 3

except:
    import Tkinter as tk  # for python 2


def start_ezadb():
    print("python version: {0} on {1}".format(
        platform.python_version(), sys.platform))
    root = tk.Tk()
    root.withdraw()
    app = EzADBGui(root)
    root.title("eZADB - Easy Android Debug Bridge")
    root.wm_resizable(False, False)

    root.deiconify()
    app.run()


if __name__ == '__main__':
    try:
        start_ezadb()
    except:
        Log.error(None, "__main__: Unexpected error={ex}".format(ex=sys.exc_info()))



import os

import configparser as ConfigParser


class Config:
    class Section(object):
        def __init__(self, section_name, config):
            self.__section_name = section_name
            self.__configParser = config

        def __str__(self):
            return '%s: %s' % (self.__section_name, self.__configParser)

        def get(self, name):
            try:
                return self.__configParser.get(self.__section_name, name)
            except Exception as n_exp:
                print('Cannot found value for %s in config: %s\n' % (name, n_exp))
                return None

        def set(self, key, value):
            try:
                if not self.__configParser.has_section(self.__section_name):
                    self.__configParser.add_section(self.__section_name)
                self.__configParser.set(self.__section_name, key, str(value))
            except Exception as n_exp:
                print('Cannot set value %s for %s to config: %s\n' % (value, key, n_exp))

    def __init__(self, config_file_path):
        self.__confFilePath = config_file_path
        self.__sections = dict()
        self.__configParser = ConfigParser.ConfigParser()
        self.__configParser.optionxform = str
        self._add_config_ini(config_file_path)

    def get_section(self, name):
        if name not in self.__sections:
            self.__sections[name] = self.Section(name, self.__configParser)
        return self.__sections[name]

    def save_changes(self):
        with open(self.__confFilePath, 'w') as configfile:
            self.__configParser.write(configfile)

    def _add_config_ini(self, config_file_path):
        # check if the file exists
        if not os.path.exists(config_file_path):
            self.__configParser.write(open(config_file_path, 'w'))
        else:
            self.__configParser.read(config_file_path)
        self.__sections = dict(
            map(lambda section: (section, self.Section(section, self.__configParser)), self.__configParser.sections()))



from os import path
from setuptools import setup, find_packages
from ezadb import __AUTHOR__
from ezadb import __version__

setup(
    name='eZadb',
    version=__version__,
    description='ezADB - Easy Android Debug Bridge. Use adb like a boss!',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python'
    ],
    long_description=open(path.join(path.abspath(path.dirname(__file__)),'docs/README.rst')).read(),
    keywords='android adb android-debug-bridge development',
    url='https://filipebatista.github.io/eZadb',
    author='{user1}'.format(user1=__AUTHOR__[0]['Name']),
    author_email='{email1}'.format(email1=__AUTHOR__[0]['Email']),
    license='GPL-3.0',
    packages=find_packages(),
    install_requires=[
        'backports.shutil_which',
        'configparser==3.5.0',
        'pygubu==0.9.8.1',
        'enum34==1.1.6',
        'pillow==4.3.0',
        'future'
    ],
    entry_points={
        'console_scripts': [
            'ezadb_cmd = ezadb.ezadb_cmd:main'
        ],
        'gui_scripts': [
            'ezadb = ezadb.__main__:start_ezadb'
        ]
    },
    include_package_data=True,
    python_requires='>=2.7',
    zip_safe=False)



from PIL import Image, ImageTk

from ezadb.utils.Utils import Utils

try:
    import tkinter as Tk  # for python 3
except:
    import Tkinter as Tk  # for python 2


class PreviewImageDialog:
    def __init__(self, parent, file_path, translator=None):
        self.translator = translator
        top = self.top = Tk.Toplevel(parent)
        # get screen width and height
        screen_width = top.winfo_screenwidth()
        screen_height = top.winfo_screenheight()

        top.attributes('-topmost', 2)
        top.title(_("ScreenShot Capture"))
        top.wm_resizable(False, False)

        # set the app icon
        Utils.set_app_icon(self.top)

        image_pil = Image.open(file_path)
        zoom_sizes = PreviewImageDialog.calculate_aspect_ratio(image_pil.width, image_pil.height,
                                                               screen_width, screen_height)
        # adjust the image to screen size
        image_pil = image_pil.resize((zoom_sizes['width'], zoom_sizes['height']), Image.ANTIALIAS)
        self.image = ImageTk.PhotoImage(image_pil)

        self.label = Tk.Label(top, image=self.image).pack()
        self.top.grab_set()

    @staticmethod
    def calculate_aspect_ratio(src_width, src_height, max_width, max_height):

        ratio = min(float(max_width) / src_width, float(max_height) / src_height)
        return {'width': int(src_width * ratio), 'height': int(src_height * ratio)}



import platform
import os
import time


class SystemUtils:

    WINDOWS_OS = 'Windows'
    LINUX_OS = 'Linux'

    @staticmethod
    def os_name():
        """ Returns the system/OS name, e.g. 'Linux' or 'Windows'
                An empty string is returned if the value cannot be determined.
            """
        return platform.system()

    @staticmethod
    def generate_random_filename(extension):
        return '{timestamp}.{ext}'.format(timestamp=int(round(time.time() * 1000)), ext=extension)

    @staticmethod
    def os_based_path(path):
        """ Retrieve the correct path operating system independent """
        return os.path.expanduser(path)
